# -*- coding: utf-8 -*-
import time
from functools import partial, wraps
from contextlib import contextmanager

from simplelog import logger

now = time.time


def fn_timer(fn=None, *, prefix="", interval=0.0):
    """
    计算 fn  的运算时间

    :param fn: 函数, callable 对象
    :param prefix: str  给函数一个标识 前缀  默认是 ""
    :param interval: float  单位是 s,  给定一个阈值,超过某一个阈值,就记录时间. 否则不记录,
                     该参数 默认值 是:0.0 ,即 记录所有的函数的运行时间.

    :return:
    """
    if fn is None:
        return partial(fn_timer, prefix=prefix, interval=interval)

    @wraps(fn)
    def function_timer(*args, **kwargs):
        start = now()
        result = fn(*args, **kwargs)
        t = now() - start
        if t >= interval:
            logger.info(f'{prefix}{fn.__name__} total running time {now() - start} seconds')
        return result

    return function_timer


@contextmanager
def code_timer(name='default'):
    """
    计算代码段的运行时间
    :param name: 用于区分不同代码段的名称
    :return:
    :usage:

     ..code-block:
         with timer('delay'):
                     time.sleep(3)
    """
    start = now()
    yield
    logger.info(f'{name} total running time {now() - start} seconds')



# -*- coding: utf-8 -*-

__version__ = "1.1.5"

__name__ = 'useful_decoration'
__author__ = 'frank'
__email__ = '15769162764@163.com'

from useful_decoration.timer import fn_timer
from useful_decoration.timer import code_timer

from useful_decoration.decorations import element_mapping

from useful_decoration.single import single
from useful_decoration.single import singleton



# -*- coding: utf-8 -*- 
"""
@User     : Frank
@File     : setup.py
@DateTime : 2019-09-16 11:24 
@Email    : 15769162764@163.com
"""
from setuptools import setup, find_packages
import io
import re

with io.open('README.rst', 'r', encoding='utf8') as f:
    long_description = f.read()

with io.open("src/useful_decoration/__init__.py", "rt", encoding="utf8") as f:
    version = re.search(r'__version__ = "(.*?)"', f.read()).group(1)

setup(
    name="useful_decoration",
    license='Apache License 2.0',
    version=version,
    packages=find_packages("src"),
    zip_safe=False,
    include_package_data=True,
    package_dir={"": "src"},
    long_description=long_description,
    url='https://github.com/changyubiao/useful_decoration',
    author='frank',
    author_email='15769162764@163.com',
    description='powerful and useful decorations',

    project_urls={
        "Documentation": "https://useful-decoration.readthedocs.io/en/latest/",
        "Code": "https://github.com/changyubiao/useful_decoration",
    },

    python_requires='>=3.6',
    install_requires=[
        "pysimple-log",

    ],

    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],

)



# -*- coding: utf-8 -*- 

"""
@Time   : 2020/4/12 19:34
@File   : single.py
@Author : 15769162764@163.com

"""
import time

from functools import update_wrapper


def single(fn):
    """
    这个函数 只会计算一次.并且对函数的参数不敏感.
    single is not  sensitive to
    argument values, and will always return the ame value if
    called with different arguments.

    :param fn:
    :return:
    """
    name = fn.__name__

    def wrapper(*args, **kw):
        if name not in single.__dict__:
            ret = fn(*args, **kw)
            single.__dict__[name] = ret
            return ret
        else:
            return single.__dict__[name]

    # copy 源数据 docstring name 等等..
    return update_wrapper(wrapper, fn)


def singleton(cls):
    """
    类装饰器 的使用,
    这样简单的可以生成一个对象
    :param cls:
    :return:
    """
    instances = dict()

    def wrapper(*args, **kwargs):
        if cls not in instances:
            instances[cls] = cls(*args, **kwargs)
        return instances[cls]

    # copy 源数据 docstring name 等等..
    return update_wrapper(wrapper, cls)


@singleton
def my_sum(a, b):
    print("my sum begin")
    # time.sleep(1)

    return a + b


@single
class Person:
    """
    This is  Person
    """
    name = "Frank"


def test_person():
    p = Person()
    p2 = Person()
    p3 = Person()

    print(p, p2, p3)

    assert p == p2 == p3

    assert id(p) == id(p2) == id(p3)


if __name__ == '__main__':
    print(my_sum(1, 4))
    print(my_sum(1, 4))
    print(my_sum(1, 4))
    print(my_sum(1, 4))

    test_person()

    pass
